/*
 * Polizeiauto.c
 *
 * Created: 18.12.2011 16:05:03
 *  Author: Chris
 */ 

#include "timing.h"
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "sirene.h"

#define MODE_SOUND 4
#define MODE_BLINKALT 1
#define MODE_BLINK 2
#define MODE_LIGHT 8

void audio_init() {
	PLLCSR = (1<<PLLE);
	while((PLLCSR & (1<<PLOCK)) == 0);
	PLLCSR |= (1<<PCKE);
	TCCR1  = (1<<PWM1A) | (1<<COM1A1) | (0<<COM1A0);
	TCCR1 |= (0<<CS13) | (1<<CS12) | (0<<CS11) | (0<<CS10);      //Prescaler 64 MHz/(8*256) -> 31.25kHz
}

volatile uint8_t partpos = 0;
volatile uint16_t samplepos = 0;
volatile uint8_t sample;

volatile uint8_t mode = 0;

inline void play() {
	OCR1A = sample;
	
	if(partpos < 8) {
		sample = pgm_read_byte(&aud_sirene1[samplepos]);
	} else {
		sample = pgm_read_byte(&aud_sirene2[samplepos]);
	}
	
	samplepos++;
		
	if(sample == 0) {
		partpos++;
		if(partpos == 16) { partpos = 0; }
		samplepos = 0;
		
		if(partpos < 8) {
			sample = pgm_read_byte(&aud_sirene1[samplepos]);
		} else {
			sample = pgm_read_byte(&aud_sirene2[samplepos]);
		}
		samplepos++;
		
	}
	
}

volatile uint8_t postscaler = 0;
volatile uint8_t postscaler_fired = 0;

ISR(TIMER0_COMPA_vect) {
	if(mode & MODE_SOUND) {
		play();
	} else {
		if(OCR1A > 0) OCR1A--;
	}

	if(postscaler == 0) {
		postscaler = 55+1;
		postscaler_fired = 1;
	}
	postscaler--;
	
}

void init_timer0() {
	//um die Samples richtig wiederzugeben wird ein zweiter Timer mit 90s Samplingrate bentigt
	TCCR0A = (1<<WGM01);
	TCCR0B = (0<<CS02) | (1<<CS01) | (0<<CS00); //Prescaler 8
	OCR0A = 180; //(F_CPU/8*90s)
	TIMSK |= (1<<OCIE0A);
}

void adc_init() {
	ADMUX = (0<<REFS1) | (0<<REFS0) | (0<<ADLAR) | (0<<REFS2) | (0<<MUX3) | (0<<MUX2) | (1<<MUX1) | (1<<MUX0);
	ADCSRA = (1<<ADEN) | (0<<ADSC) | (0<<ADATE) | (0<<ADIF) | (0<<ADIE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
}

inline uint8_t adc_read() {
	ADCSRA |= (1 << ADSC);
	while(!(ADCSRA & (1 << ADSC)));
	return ADC>>6;
}

int main(void) {
	
	DDRB |= (1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB4);
	
	audio_init();
	init_timer0();
	
	timing_init();
	adc_init();
	sei();
	uint8_t adc_tick = 20;
	
	while(1) {
		if(postscaler_fired) {
			//alle ~5ms
			postscaler_fired = 0;
			timing_tick();
			
			adc_tick--;
			if(adc_tick == 0) {
				adc_tick = 20;
				mode = adc_read();
				if(mode & MODE_LIGHT) {
					PORTB |= 1<<PB4;
				} else {
					PORTB &= ~(1<<PB4);
				}
				
				if(mode & MODE_BLINK) {
					if(mode & MODE_BLINKALT) {
						timing_setpattern(1);
					} else {
						timing_setpattern(2);
					}
				} else {
					timing_setpattern(0); //Aus
				}
			}
		}
	}
}


